package com.ruoyi.yh.report.service.impl;

import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.system.user.domain.SysUserQuery;
import com.ruoyi.system.user.domain.SysUserVO;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.yh.price.domain.PriceQuery;
import com.ruoyi.yh.price.domain.PriceVO;
import com.ruoyi.yh.price.domain.enums.PriceStatus;
import com.ruoyi.yh.price.service.PriceService;
import com.ruoyi.yh.prodOrder.domain.ProdOrderQuery;
import com.ruoyi.yh.prodOrder.domain.ProdOrderVO;
import com.ruoyi.yh.prodOrder.service.ProdOrderService;
import com.ruoyi.yh.report.domain.ReportVO;
import com.ruoyi.yh.report.domain.bo.ReportBO;
import com.ruoyi.yh.report.domain.enums.ReportStatus;
import com.ruoyi.yh.report.service.ReportService;
import com.ruoyi.yh.report.service.ReportValidator;
import com.ruoyi.yh.reportOrderProd.domain.bo.ReportOrderProdBO;
import com.ruoyi.yh.reportProd.domain.bo.ReportProdBO;
import com.ruoyi.yh.reportUserProd.domain.bo.ReportUserProdBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author wjh
 * 2024/11/8
 */
@Service
public class ReportValidatorImpl implements ReportValidator {

    @Autowired
    private ReportService reportService;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private PriceService priceService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ProdOrderService prodOrderService;

    @Override
    public void checkPreEdit(ReportBO bo) {
        ServiceUtil.assertion(bo == null || bo.getReportId() == null, "参数错误");

        ReportVO old = reportService.selectReportByReportId(bo.getReportId());
        ServiceUtil.assertion(old == null || old.getReportId() == null, "待更新的报表不存在");
        ServiceUtil.assertion(!ReportStatus.canEdit().contains(old.getStatus()), "报表当前状态不允许更新，请刷新后重试");

        // 校验部门
        this.checkDept(bo.getDeptId());

        // 校验产量列表
        this.checkProduct(bo.getProductList());

    }

    @Override
    public void afterCheck(ReportVO vo) {

        // TODO 工序是否当前部门的工序

        // TODO 员工是否当前部门的员工

        // TODO 订单是否当前部门订单
    }

    private void checkProduct(List<ReportProdBO> productList) {
        if (CollectionUtils.isEmptyElement(productList)) {
            return;
        }

        PriceQuery query = new PriceQuery();
        query.setPriceIds(CollectionUtils.map(productList, ReportProdBO::getPriceId));
        query.setNeedScope(true);
        List<PriceVO> priceList = priceService.selectPriceList(query);

        for (ReportProdBO prod : productList) {
            if (prod == null || prod.getPriceId() == null) {
                continue;
            }
            // 校验工序
            PriceVO price = priceList.stream().filter(item -> Objects.equals(item.getPriceId(), prod.getPriceId())).findFirst().orElse(null);
            ServiceUtil.assertion(price == null, "工序%s不存在", prod.getPriceName());
            ServiceUtil.assertion(!PriceStatus.canCheckToReport().contains(price.getStatus()), "工序%s当前状态不允许被使用", price.getName());
            ServiceUtil.assertion(price.getDisabled() == null || price.getDisabled(), "工序%s已被禁用，无法使用", price.getName());

            // 校验产量和工序的数值是否一致

        }

        // 员工是否当前用户可选
        List<ReportUserProdBO> userProdList = productList.stream().map(ReportProdBO::getUserProdList).flatMap(List::stream).collect(Collectors.toList());
        SysUserQuery userQuery = new SysUserQuery();
        userQuery.setUserIds(CollectionUtils.map(userProdList, ReportUserProdBO::getUserId));
        userQuery.setNeedScope(true);
        List<SysUserVO> userList = userService.selectUserListScope(userQuery);
        for (ReportUserProdBO userProd : userProdList) {
            if (userProd == null || userProd.getUserId() == null) {
                continue;
            }
            // 校验员工
            SysUserVO user = userList.stream().filter(item -> Objects.equals(item.getUserId(), userProd.getUserId())).findFirst().orElse(null);
            ServiceUtil.assertion(user == null, "员工不存在");
        }

        // 订单是否当前用户可选
        List<ReportOrderProdBO> orderProdList = productList.stream().map(ReportProdBO::getOrderProdList).flatMap(List::stream).collect(Collectors.toList());
        ProdOrderQuery orderQuery = new ProdOrderQuery();
        orderQuery.setIds(CollectionUtils.map(orderProdList, ReportOrderProdBO::getOrderId));
        orderQuery.setNeedScope(true);
        List<ProdOrderVO> orderList = prodOrderService.selectProdOrderList(orderQuery);
        for (ReportOrderProdBO orderProd : orderProdList) {
            if (orderProd == null || orderProd.getOrderId() == null) {
                continue;
            }
            // 校验订单
            ProdOrderVO order = orderList.stream().filter(item -> Objects.equals(item.getId(), orderProd.getOrderId())).findFirst().orElse(null);
            ServiceUtil.assertion(order == null, "生产订单%s不存在");
        }

    }

    private void checkDept(Long deptId) {
        if (deptId == null) {
            return;
        }

        // 部门是否当前用户可选
        SysDept dept = deptService.selectDeptById(deptId, true);
        ServiceUtil.assertion(dept == null, "部门不存在");
    }
}
